#include "General.h"
#include "MixFileFactoryClass.h"
#include "SimpleFileFactoryClass.h"
#include "Crc32.h"

struct FileStruct
{
	const char *filename;
	unsigned long CRC;
	bool operator== (const FileStruct &src)
	{
		return false;
	}
	bool operator!= (const FileStruct &src)
	{
		return true;
	}
};

DynamicVectorClass<FileStruct> Filenames;

void ParseFolder(const char *basepath,const char *path)
{
	StringClass str;
	if (path)
	{
		str.Format("%s\\%s\\*",basepath,path);
	}
	else
	{
		str.Format("%s\\*",basepath);
	}
	WIN32_FIND_DATAA data;
	HANDLE handle = ::FindFirstFile (str, &data);
	if (handle != INVALID_HANDLE_VALUE)
	{
		do
		{
			StringClass str2;
			if (path)
			{
				str2.Format("%s\\%s",path,data.cFileName);
			}
			else
			{
				str2.Format("%s",data.cFileName);
			}
			if (data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
				if (strcmp(data.cFileName, ".") && strcmp(data.cFileName, ".."))
				{
					ParseFolder(basepath,str2);
				}
			}
			else
			{
				FileStruct fs;
				fs.filename = newstr(str2);
				fs.CRC = CRC_Stringi(str2);
				Filenames.Add(fs);
			}
		}
		while (::FindNextFile (handle, &data));
		::FindClose (handle);
	}
}

int File_Info_Compare(const void *v1, const void *v2)
{
	unsigned long crc1 = ((FileStruct *)v1)->CRC;
	unsigned long crc2 = ((FileStruct *)v2)->CRC;
	int ret = -1;
	if (crc1 >= crc2)
	{
		ret = crc1 > crc2;
	}
	return ret;
}

int main(int argc, char* argv[])
{
	if (argc < 3)
	{
		printf("makemix tool to generate a renegade mix file from a folder full of files");
		printf("usage:\n");
		printf("makemix filename folder\n");
		printf("filename is the name of the mix file to create\n");
		printf("folder is the path to the folder containing the files to add\n");
		printf("The mix file will be created in the folder containing makemix.exe\n");
		printf("makemix will put every file in the specified folder into the mix file, including those in subfolders\n");
		return 1;
	}
	char path[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, path);
	SetCurrentDirectory(argv[1]);
	ParseFolder(argv[1],0);
	qsort(&Filenames[0],Filenames.Count(),sizeof(FileStruct),File_Info_Compare);
	StringClass mp;
	mp.Format("%s\\%s",path,argv[2]);
	MixFileCreator mix(mp);
	for (int i = 0;i < Filenames.Count();i++)
	{
		StringClass str;
		str.Format("%s\\%s",argv[1],Filenames[i].filename);
		mix.Add_File(str,Filenames[i].filename);
	}
	return 0;
}
