/*	Renegade Scripts.dll
	Copyright 2011 Tiberian Technologies

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/
// Include files
#include "general.h"
#include "scripts.h"
#include "engine.h"
#include "dp88_customAI.h"
#include "VehicleGameObj.h"

/*------------------------
Base class for custom AI's
--------------------------*/
void dp88_customAI::Created( GameObject *obj )
{
	Init(obj);
	loadSettings(obj);
}

void dp88_customAI::Action_Complete( GameObject *obj, int action_id, ActionCompleteReason complete_reason )
{
	if ( action_id == 2 )
	{
		targetID = 0;
		targetPriority = 0.0f;
		Commands->Action_Reset( obj, 100 );
	}
}


void dp88_customAI::Init( GameObject *obj )
{
	// Set default values
	targetID = 0;
	targetLastSeen = 0;
	targetPriority = 0.0f;
	primary_target = true;

	// Set state
	Commands->Enable_Hibernation( obj, false );
	Commands->Innate_Enable( obj );
	Commands->Enable_Enemy_Seen( obj, true );
	Commands->Enable_Vehicle_Transitions( obj, false );
}

void dp88_customAI::loadSettings ( GameObject *obj, bool loadSecondaryFireSettings, bool loadBuildingTargetSettings, bool oldSettingNames )
{
	// Load base target type priorities
	priority_infantry		= ( oldSettingNames ) ? Get_Int_Parameter ( "Priority_Infantry" )		: Get_Float_Parameter ( "Priority_Infantry" );
	priority_lightVehicle	= ( oldSettingNames ) ? Get_Int_Parameter ( "priority_lightVehicle" )	: Get_Float_Parameter ( "Priority_Light_Vehicle" );
	priority_heavyVehicle	= ( oldSettingNames ) ? Get_Int_Parameter ( "priority_heavyVehicle" )	: Get_Float_Parameter ( "Priority_Heavy_Vehicle" );
	priority_VTOL			= ( oldSettingNames ) ? Get_Int_Parameter ( "Priority_VTOL" )			: Get_Float_Parameter ( "Priority_VTOL" );
	priority_building		= ( loadBuildingTargetSettings ) ? (( oldSettingNames ) ? Get_Int_Parameter ( "Priority_Building" ) : Get_Float_Parameter ( "Priority_Building" )) : 0;
	

	// Load priority modifiers
	modifier_distance		= ( oldSettingNames ) ? 0.25f	: Get_Float_Parameter("Modifier_Distance");
	modifier_target_damage	= ( oldSettingNames ) ? 0.1f	: Get_Float_Parameter("Modifier_Target_Damage");
	modifier_target_value	= ( oldSettingNames ) ? 0.05f	: Get_Float_Parameter("Modifier_Target_Value");


	// Load primary / secondary fire states for target types
	primary_infantry		= ( loadSecondaryFireSettings ) ? (( Get_Int_Parameter ( "Weapon_Infantry" ) == 2 )	? false : true ) : true;
	primary_lightVehicle	= ( loadSecondaryFireSettings ) ? (( (oldSettingNames && Get_Int_Parameter ( "Weapon_Vehicle" ) == 2) || (!oldSettingNames && Get_Int_Parameter("Weapon_Light_Vehicle") == 2) )	? false : true ) : true;
	primary_heavyVehicle	= ( loadSecondaryFireSettings ) ? (( (oldSettingNames && Get_Int_Parameter ( "Weapon_Vehicle" ) == 2) || (!oldSettingNames && Get_Int_Parameter("Weapon_Heavy_Vehicle") == 2) )	? false : true ) : true;
	primary_VTOL			= ( loadSecondaryFireSettings ) ? (( Get_Int_Parameter ( "Weapon_VTOL" ) == 2 )		? false : true ) : true;
	primary_building		= ( loadSecondaryFireSettings ) ? (( loadBuildingTargetSettings ) ? (( Get_Int_Parameter ( "Weapon_Building" ) == 2 )	? false : true) : true ) : true;


	// Get min and max ranges
	primary_minRange	= Get_Int_Parameter ( "Min_Attack_Range" );
	primary_maxRange	= Get_Int_Parameter ( "Max_Attack_Range" );
	secondary_minRange	= ( loadSecondaryFireSettings ) ? Get_Int_Parameter ( "Min_Attack_Range_Secondary" ) : 0;
	secondary_maxRange	= ( loadSecondaryFireSettings ) ? Get_Int_Parameter ( "Max_Attack_Range_Secondary" ) : 0;


	// If debugging is enabled open the debug file
	debug = ( Get_Int_Parameter("Debug") == 1 );
	if ( debug )
	{
		StringClass filename(true);
		filename.Format ( "%s_%d_%d.log", Commands->Get_Preset_Name(obj), Commands->Get_ID(obj), (int)time(NULL) );
		debugFile = fopen(filename.Peek_Buffer(),"w");
	}
}

float dp88_customAI::getDistance ( GameObject *obj1, GameObject *obj2 )
{
	// Gets distance between two objects
	return Commands->Get_Distance ( Commands->Get_Position ( obj1 ), Commands->Get_Position ( obj2 ) );
}

float dp88_customAI::getPriority( GameObject *obj, GameObject *target )
{
	if ( !obj || !target )
		return 0.0f;

	if ( debug ) fprintf ( debugFile, "Calculating priority of %d (%s)\n", Commands->Get_ID(target), Commands->Get_Preset_Name(target) );

	// Priority is 0 if the unit is dead or not on teams 0 or 1
	if ( ( Commands->Get_Health ( target ) + Commands->Get_Shield_Strength ( target ) ) == 0
		|| ( Commands->Get_Player_Type ( target ) != 0 && Commands->Get_Player_Type ( target ) != 1 ) )
	{
		if ( debug ) fprintf ( debugFile, "Target %d is dead or unteamed, ignoring\n", Commands->Get_ID(target));
		return 0.0;
	}

	if ( Is_Vehicle(target) && !IsVehicleAIEnabled(target->As_VehicleGameObj()) && IsVehicleEmpty(target->As_VehicleGameObj()) )
	{
		if ( debug ) fprintf ( debugFile, "Target %d is an empty, non-AI vehicle, ignoring\n", Commands->Get_ID(target));
		return 0.0;
	}

	float priority = 0.0;


	// Assign base priority
	if ( Is_Soldier ( target ) && !Get_Fly_Mode(target) )
		priority = (float)priority_infantry;
	else if ( Is_VTOLVehicle ( target ) || (Is_Soldier(target) && Get_Fly_Mode(target)) )
		priority = (float)priority_VTOL;
	else if ( Is_Vehicle ( target ) && Is_Script_Attached ( target, "dp88_AI_heavyVehicleMarker" ) )
		priority = (float)priority_heavyVehicle;
	else if ( Is_Vehicle ( target ) )
		priority = (float)priority_lightVehicle;
	else if ( Is_Building ( target ) )
		priority = (float)priority_building;

	// Return if our base priority for this target type is 0
	if ( priority == 0.0f )
	{
		if ( debug ) fprintf ( debugFile, "\tInvalid target type, priority is 0.0\n" );
		return 0.0f;
	}

	if ( debug ) fprintf ( debugFile, "\tBase priority for this unit type is %.2f\n", priority );


	// Apply target value modifier (add X priority for every 10 credits the target
	// cost to purchase - not applicable to buildings)
	if ( !Is_Building( target ) )
	{
		int cost = Get_Team_Cost( Commands->Get_Preset_Name ( target ), Commands->Get_Player_Type ( target ) );
		priority += ((float)cost/10.0f) * modifier_target_value;

		if ( debug ) fprintf ( debugFile, "\tTarget Value Modifier: Adding %.4f priority ((Value %d /10) * modifier %.2f)\n", ((float)cost/10.0f)*modifier_target_value, cost, modifier_target_value );
	}


	// Apply target damage modifier (add X priority for 10 points of damage the
	// target has already taken from their maximum health/armour)
	float totalHitPts = Commands->Get_Health ( target ) + Commands->Get_Shield_Strength ( target );
	float maxHitPts = Commands->Get_Max_Health ( target ) + Commands->Get_Max_Shield_Strength ( target );
	priority += ((maxHitPts-totalHitPts)/10) * modifier_target_damage;

	if ( debug ) fprintf ( debugFile, "\tTarget Damage Modifier: Adding %.4f priority ((Target Damage Taken %.2f) /10) * modifier %.2f)\n", ((maxHitPts-totalHitPts)/10)*modifier_target_damage, maxHitPts-totalHitPts, modifier_target_damage );


	// Apply target distance modifier (subtract X distance for every 10 units of
	//  distancebetween this object and the target)
	float distance = Commands->Get_Distance ( Commands->Get_Position ( obj ), Commands->Get_Position ( target ) );
	priority -= (distance/10) * modifier_distance;

	if ( debug ) fprintf ( debugFile, "\tDistance Modifier: Subtracting %.4f priority ((Distance to target %.2f) /10) * modifier %.2f)\n", (distance/10)*modifier_distance, distance, modifier_distance );


	// Ensure priority is above 0
	if ( debug ) fprintf ( debugFile, "\tFinal Priority: %.4f\n", priority );
	if ( priority <= 0.0 )
	{
		priority = 0.0001f;
		if ( debug ) fprintf ( debugFile, "\tAdjusting final priority to 0.0001 (must be >0)\n" );
	}


	// Return result
	return priority;
}


float dp88_customAI::getPriority ( GameObject *obj, int target_id )
{
	// Overloaded function to accept object ID's
	return getPriority ( obj, Commands->Find_Object ( target_id ) );
}


bool dp88_customAI::getPrimary ( GameObject *target )
{
	if ( Is_Soldier ( target ) )
		return primary_infantry;
	else if ( Is_VTOLVehicle ( target ) )
		return primary_VTOL;
	else if ( Is_Vehicle ( target ) )
		return (Is_Script_Attached(target,"dp88_AI_heavyVehicleMarker")) ? primary_heavyVehicle : primary_lightVehicle;
	else if ( Is_Building ( target ) )
		return primary_building;
	return true;
}


bool dp88_customAI::IsVehicleEmpty( VehicleGameObj* vobj )
{
	if ( vobj == NULL )
		return false;
	return (vobj->Get_Occupant_Count() == 0);
}


bool dp88_customAI::IsVehicleAIEnabled( VehicleGameObj* vobj )
{
	if ( vobj == NULL )
		return false;
	return (vobj->Get_Action()->Is_Acting());
}



/*------------------------
Offensive Tank AI
--------------------------*/

void dp88_tankAI_Offensive::Created( GameObject *obj )
{
	Init(obj);

	// Load settings - using old setting names
	loadSettings( obj, true, true, true );


	// Set unit specific settings
	movingToTarget = false;

	primary_prefRange = Get_Int_Parameter ( "Preferred_Attack_Range" );
	secondary_prefRange = Get_Int_Parameter ( "Preferred_Attack_Range_Secondary" );

	// Start timer which runs for the lifetime of this object
	Commands->Start_Timer ( obj, this, 1.0, TIMER_CHECK_TARGET );
}


void dp88_tankAI_Offensive::Enemy_Seen ( GameObject *obj, GameObject *enemy )
{
	// If we saw a soldier in a vehicle, switch the GameObject to that of the vehicle
	GameObject *o = Get_Vehicle(enemy);
	if ( o )
		enemy = o;

	//fprintf( debugFile, "Seen enemy %s\n", Commands->Get_Preset_Name(enemy) );

	// If this is our current target update it's last seen time and priority
	if ( Commands->Get_ID ( enemy ) == targetID )
	{
		targetLastSeen = (int)time(NULL);
		targetPriority = getPriority(obj, targetID);
		return;
	}

	// Get priority for seen object and return if 0
	float seenPriority = getPriority( obj, enemy );
	if ( seenPriority == 0.0 )
		return;

	// Decide if we want to use primary or secondary fire
	bool attackPrimary = getPrimary ( enemy );
	
	// Get distance to target, and return if they are too close
	float targetDistance = getDistance ( obj, enemy );
	if ( (attackPrimary && targetDistance < primary_minRange) || (!attackPrimary && targetDistance < secondary_minRange) )
		return;

	//fprintf( debugFile, "Range to enemy: %d\n", (int)targetDistance );
	// Work out its targeting priority, to see if we should shoot it
	if ( targetID == 0 || seenPriority > targetPriority )
	{
		//fprintf( debugFile, "Higher priority than existing target... attacking!\n" );

		ActionParamsStruct params;
		params.Set_Basic( this, 100, 2 );
		
		if ( attackPrimary && targetDistance < primary_maxRange )
		{
			targetID = Commands->Get_ID ( enemy );
			primary_target = attackPrimary;
			targetLastSeen = (int)time(NULL);

			if ( targetDistance > primary_prefRange )
			{
				movingToTarget = true;
				params.Set_Movement(enemy, 1.0f, (float)primary_prefRange );
			}
			params.Set_Attack( enemy, (float)primary_maxRange, 0.0, true);
			params.AttackCheckBlocked = false;
			Commands->Action_Attack( obj, params );
		}
		else if ( !attackPrimary && targetDistance < secondary_maxRange )
		{
			targetID = Commands->Get_ID ( enemy );
			primary_target = attackPrimary;
			targetLastSeen = (int)time(NULL);
		
			if ( targetDistance > secondary_prefRange )
			{
				movingToTarget = true;
				params.Set_Movement( enemy, 1.0f, (float)secondary_prefRange );
			}
			params.Set_Attack( enemy, (float)secondary_maxRange, 0.0, false);
			params.AttackCheckBlocked = false;
			Commands->Action_Attack( obj, params );
		}
	}
}

void dp88_tankAI_Offensive::Timer_Expired( GameObject *obj, int number )
{
	// Check target is still alive, still an enemy (eg: stolen vehicles) and in range
	if ( number == TIMER_CHECK_TARGET )
	{
		if ( targetID != 0 )
		{
			//fprintf( debugFile, "Checking target validity...\n" );
			GameObject *target = Commands->Find_Object ( targetID );

			if ( ( Commands->Get_Health ( target ) + Commands->Get_Shield_Strength ( target ) <= 0 )
				|| Commands->Get_Player_Type ( target ) == 2
				|| Commands->Get_Player_Type ( target ) == Commands->Get_Player_Type ( obj )
				|| ( primary_target && getDistance ( obj, target ) > primary_maxRange )
				|| ( primary_target && getDistance ( obj, target ) < primary_minRange )
				|| ( !primary_target && getDistance ( obj, target ) > secondary_maxRange )
				|| ( !primary_target && getDistance ( obj, target ) < secondary_minRange )
				|| (int)time(NULL) - targetLastSeen > 3 )	// Not seen target for 3 seconds
			{
				//fprintf( debugFile, "Target invalid, aborting.. \n" );
				targetID = 0;
				Commands->Action_Reset ( obj, 100 );
			}

			// Stop chasing if we have reached preferred range
			else if ( movingToTarget && primary_target && getDistance ( obj, target ) <= primary_prefRange )
			{
				Commands->Action_Reset ( obj, 100 );
				movingToTarget = false;

				ActionParamsStruct params;
				params.Set_Basic( this, 100, 2 );
				params.Set_Attack( target, (float)primary_maxRange, 0.0, true);
				params.AttackCheckBlocked = false;
				Commands->Action_Attack ( obj, params );
			}

			else if ( movingToTarget && !primary_target && getDistance ( obj, target ) <= secondary_prefRange )
			{
				Commands->Action_Reset ( obj, 100 );
				movingToTarget = false;

				ActionParamsStruct params;
				params.Set_Basic( this, 100, 2 );
				params.Set_Attack( target, (float)secondary_maxRange, 0.0, false);
				params.AttackCheckBlocked = false;
				Commands->Action_Attack ( obj, params );
			}

			// Chase them if they are moving away from us...
			else if ( !movingToTarget && primary_target && getDistance ( obj, target ) > primary_prefRange )
			{
				Commands->Action_Reset ( obj, 100 );
				movingToTarget = true;

				ActionParamsStruct params;
				params.Set_Basic( this, 100, 2 );
				params.Set_Movement( target, 1.0f, (float)primary_prefRange );
				params.Set_Attack( target, (float)primary_maxRange, 0.0, true);
				params.AttackCheckBlocked = false;
				Commands->Action_Attack ( obj, params );
			}

			// Chase them if they are moving away from us...
			else if ( !movingToTarget && !primary_target && getDistance ( obj, target ) > secondary_prefRange )
			{
				Commands->Action_Reset ( obj, 100 );
				movingToTarget = true;

				ActionParamsStruct params;
				params.Set_Basic( this, 100, 2 );
				params.Set_Movement( target, 1.0f, (float)secondary_prefRange );
				params.Set_Attack( target, (float)secondary_maxRange, 0.0, false);
				params.AttackCheckBlocked = false;
				Commands->Action_Attack ( obj, params );
			}
		}

		// Timer always runs
		Commands->Start_Timer ( obj, this, 1.0, TIMER_CHECK_TARGET );
	}
}







/*------------------------
Turret AI
--------------------------*/

void dp88_AI_Turret::Created( GameObject *obj )
{
	Init(obj);
	loadSettings(obj);
}

void dp88_AI_Turret::Init( GameObject *obj )
{
	dp88_customAI::Init( obj );

	// Start timer which runs for the lifetime of this object
	Commands->Start_Timer ( obj, this, 1.0, TIMER_CHECK_TARGET );
}

void dp88_AI_Turret::loadSettings(  GameObject *obj, bool loadSecondaryFireSettings, bool loadBuildingTargetSettings )
{
	dp88_customAI::loadSettings(obj, loadSecondaryFireSettings, loadBuildingTargetSettings);
	requiresPower = (Get_Int_Parameter("Requires_Power") > 0 ) ? true: false;
	splashInfantry = (Get_Int_Parameter("Splash_Infantry") > 0 ) ? true: false;
}

void dp88_AI_Turret::Enemy_Seen ( GameObject *obj, GameObject *enemy )
{
	// If we saw a soldier in a vehicle, switch the GameObject to that of the vehicle
	enemy = ( Get_Vehicle(enemy) ) ? Get_Vehicle(enemy) : enemy;
	if ( debug ) fprintf( debugFile, "Seen an enemy: %d (%s)\n", Commands->Get_ID(enemy), Commands->Get_Preset_Name(enemy) );

	// If this is our current target update last seen time and priority
	if ( Commands->Get_ID ( enemy ) == targetID )
	{
		if ( debug ) fprintf( debugFile, "Enemy is current target, last seen time and priority updated\n" );
		targetLastSeen = (int)time(NULL);
		targetPriority = getPriority(obj, enemy);
		return;
	}

	// If base power is offline and we require power then abort
	if ( !checkPowerState(obj) )
	{
		if ( debug ) fprintf( debugFile, "Unable to attack, no power available..." );
		return;
	}


	// Determine if we want to use primary or secondary fire on this target
	bool attackPrimary = getPrimary ( enemy );
	if ( debug ) fprintf( debugFile, "Using %s fire on this target type\n", (attackPrimary)?"primary":"secondary" );


	// Check if the target is between our min and max firing ranges
	if ( !checkRange(obj, enemy, attackPrimary ) )
	{
		if ( debug ) fprintf( debugFile, "Unable to attack, target out of range\n" );
		return;
	}


	// Determine the priority of the target
	float enemyPriority = getPriority( obj, enemy );
	if ( enemyPriority == 0.0 )
		return;


	// If this is a higher priority than our current target attack it
	if ( targetID == 0 || enemyPriority > targetPriority )
	{
		if ( debug && targetID == 0 ) fprintf ( debugFile, "No current target to compare with, attacking enemy!\n" );
		else if ( debug ) fprintf ( debugFile, "New enemy has a higher priority than current target, attacking enemy\n" );

		targetID = Commands->Get_ID ( enemy );
		targetLastSeen = (int)time(NULL);
		targetPriority = enemyPriority;
		primary_target = attackPrimary;

		if ( splashInfantry && Is_Soldier(enemy) )
		{
			if ( debug ) fprintf ( debugFile, "Using splash damage against this target\n" );
			attackLocation(obj,Commands->Get_Position(enemy),primary_target);
		}
		else
			attackTarget(obj,enemy,primary_target);
	}
	else if ( debug ) fprintf ( debugFile, "Current target has a higher priority than new enemy, continuing to attack current target\n" );
}

void dp88_AI_Turret::Timer_Expired( GameObject *obj, int number )
{
	// Check target is still alive, still an enemy (eg: stolen vehicles) and in range
	if ( number == TIMER_CHECK_TARGET )
	{
		if ( targetID != 0 )
		{
			GameObject *target = Commands->Find_Object ( targetID );

			if ( !checkPowerState(obj)
				|| ( Commands->Get_Health ( target ) + Commands->Get_Shield_Strength ( target ) <= 0 )
				|| !checkTeam ( obj, target )
				|| !checkRange ( obj, target, primary_target )
				|| (int)time(NULL) - targetLastSeen > 3
				|| ( Is_Vehicle(target) && !IsVehicleAIEnabled(target->As_VehicleGameObj()) && IsVehicleEmpty(target->As_VehicleGameObj()) ) )
			{
				if ( debug ) fprintf ( debugFile, "Target %d no longer valid, ceasing attack.\n", targetID);
				targetID = 0;
				targetPriority = 0.0f;
				targetLastSeen = 0;
				stopAttacking(obj);
			}

			// Otherwise we are OK to continue attacking... if this is an infantry unit
			// and splash targetting is enabled then update targetting position
			else if ( splashInfantry && Is_Soldier(target) )
				attackLocation ( obj, Commands->Get_Position(target), primary_target );
		}

		// Timer always runs
		Commands->Start_Timer ( obj, this, 1.0, TIMER_CHECK_TARGET );
	}
}




// Returns true if the target is on the enemy team, false otherwise
bool dp88_AI_Turret::checkTeam ( GameObject* obj, GameObject* target )
{
	return !( Commands->Get_Player_Type ( target ) == 2 || Commands->Get_Player_Type ( target ) == Commands->Get_Player_Type ( obj ) );
}




// Returns true if the target is between our minimum and maximum range for either primary
// or secondary weapons
bool dp88_AI_Turret::checkRange ( GameObject* obj, GameObject* target, bool primary )
{
	// Get distance to target
	float distance = getDistance ( obj, target );
	if ( (primary && distance < primary_minRange) || (!primary && distance < secondary_minRange) )
	{
		if ( debug ) fprintf( debugFile, "Target is too close to attack (distance: %.3f < min range: %d)\n", distance, (primary)?primary_minRange:secondary_minRange );
		return false;
	}
	if ( (primary && distance > primary_maxRange) || (!primary && distance > secondary_maxRange) )
	{
		if ( debug ) fprintf( debugFile, "Target is too far away to attack (distance: %.3f > max range: %d)\n", distance, (primary)?primary_maxRange:secondary_maxRange );
		return false;
	}

	return true;
}


// Returns true if the base is powered or the defence does not require power
bool dp88_AI_Turret::checkPowerState ( GameObject* obj )
{
	return ( !requiresPower || Is_Base_Powered(Get_Object_Type(obj)) );
}




// These is called when a valid target has been identified and selected as the highest priority
// target, the turret should begin attacking in this function. Note that this may be called
// while another attack is already in progress, either normal or splash.
void dp88_AI_Turret::attackTarget ( GameObject* obj, GameObject* target, bool primary )
{
	// Reset any current attack (note: specifically use the function from THIS
	// class and not a derived class as we do not know what functionality may
	// be implemented in a derived class)
	dp88_AI_Turret::stopAttacking(obj);

	ActionParamsStruct params;
	params.Set_Basic( this, 100.0f, 8950 );
	params.Set_Attack( target, (primary)?(float)primary_maxRange:(float)secondary_maxRange, 0.0, primary);
	params.AttackCheckBlocked = false;
	Commands->Action_Attack( obj, params );
}

// This is called instead of attackTarget when we are set to splash infantry instead of shooting
// at them directly. Whilst we are attacking an infantry unit with splash this will be called
// regularly on a timer to update the location of the target. Note that this may be called
// while another attack is already in progress, either normal or splash.
void dp88_AI_Turret::attackLocation ( GameObject* obj, Vector3 location, bool primary )
{
	ActionParamsStruct params;
	Commands->Get_Action_Params(obj,params);
	
	// If we are already using splash on a target then we just want to modify the action (if
	// priority is 0 the action has been completed or reset, so we don't want to use modify)
	if ( params.ActionID == 8951 && params.Priority > 0.0f )
	{
		Commands->Get_Action_Params(obj,params);
		params.Set_Attack( location, (primary)?(float)primary_maxRange:(float)secondary_maxRange, 0.0, primary);
		Commands->Modify_Action(obj,8951,params,false,true);
	}

	// Otherwise setup a new action
	else
	{
		// Reset any current attack (note: specifically use the function from THIS
		// class and not a derived class as we do not know what functionality may
		// be implemented in a derived class)
		dp88_AI_Turret::stopAttacking(obj);

		params.Set_Basic( this, 100.0f, 8951 );
		params.Set_Attack( location, (primary)?(float)primary_maxRange:(float)secondary_maxRange, 0.0, primary);
		params.AttackCheckBlocked = false;
		Commands->Action_Attack( obj, params );
	}
}

// This is called when the target is no longer valid and the turret should stop attacking, this
// is called to stop both attackTarget and splashLocation attacks.
void dp88_AI_Turret::stopAttacking ( GameObject* obj )
{
	Commands->Action_Reset(obj,101.0f);
}







/*------------------------
Script Registrants
--------------------------*/

ScriptRegistrant<dp88_tankAI_Offensive> dp88_tankAI_Offensive_Registrant("dp88_tankAI_Offensive","Priority_Infantry=1:int,Weapon_Infantry=1:int,priority_lightVehicle=5:int,priority_heavyVehicle=7:int,Weapon_Vehicle=1:int,Priority_VTOL=5:int,Weapon_VTOL=1:int,Priority_Building=12:int,Weapon_Building=1:int,Max_Attack_Range=150:int,Min_Attack_Range=0:int,Preferred_Attack_Range=60:int,Max_Attack_Range_Secondary=150:int,Min_Attack_Range_Secondary=0:int,Preferred_Attack_Range_Secondary=60:int");
ScriptRegistrant<dp88_AI_Turret> dp88_AI_Turret_Registrant("dp88_AI_Turret","Priority_Infantry=1.0:float,Weapon_Infantry=1:int,Splash_Infantry=0:int,Priority_Light_Vehicle=5.0:float,Weapon_Light_Vehicle=5:int,Priority_Heavy_Vehicle=7.0:float,Weapon_Heavy_Vehicle=1:int,Priority_VTOL=5.0:float,Weapon_VTOL=1:int,Min_Attack_Range=0:int,Max_Attack_Range=150:int,Min_Attack_Range_Secondary=0:int,Max_Attack_Range_Secondary=150:int,Modifier_Distance=0.25:float,Modifier_Target_Damage=0.1:float,Modifier_Target_Value=0.05:float,Requires_Power=0:int,Debug=0:int");

// Heavy vehicle marker (dummy script)
ScriptRegistrant<dp88_AI_heavyVehicleMarker> dp88_AI_heavyVehicleMarker_Registrant("dp88_AI_heavyVehicleMarker","");