/*	Renegade Scripts.dll
	Copyright 2011 Tiberian Technologies

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/
#include "general.h"

#include "scripts.h"
#include "PhysClass.h"
#include "MoveablePhysClass.h"
#include "RenderObjClass.h"
#include "HTreeClass.h"
#include "engine_obj.h"
#include "PhysicalGameObj.h"
#include "engine_def.h"
#include "slist.h"
#include "GameObjManager.h"
#include "engine_obj2.h"
#include "SmartGameObj.h"
#include "VehicleGameObj.h"
const char SCRIPTS_API *Get_Model(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return "DUMMY";
	}
	PhysicalGameObj *o = (PhysicalGameObj *)As_PhysicalGameObj(obj);
	if (!o)
	{
		return "DUMMY";
	}
	return o->Peek_Physical_Object()->Peek_Model()->Get_Name();
}

bool SCRIPTS_API Is_TrackedVehicle(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x900C)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_VTOLVehicle(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x900D)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_WheeledVehicle(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9006)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_Motorcycle(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9002)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_Door(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9080)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_Elevator(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9081)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_DamageableStaticPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9082)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_AccessablePhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9010)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_DecorationPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9000)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_HumanPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9001)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_MotorVehicle(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9003)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_Phys3(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9004)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_RigidBody(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9005)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_ShakeableStatricPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x900F)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_StaticAnimPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9008)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_StaticPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9007)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_TimedDecorationPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x900A)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_VehiclePhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x900B)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_DynamicAnimPhys(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x900E)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_BuildingAggregate(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9083)
	{
		return false;
	}
	return true;
}

bool SCRIPTS_API Is_Projectile(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return false;
	}
	GameObject *o = As_PhysicalGameObj(obj);
	if (!o)
	{
		return false;
	}
	DefinitionClass* definition = Get_Phys_Definition(o);
	if (!definition)
	{
		return false;
	}
	int Class_ID = definition->Get_Class_ID();
	if (Class_ID != 0x9009)
	{
		return false;
	}
	return true;
}

//doesnt work for CinematicGameObjs
float SCRIPTS_API Get_Animation_Frame(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return 0;
	}
	PhysicalGameObj *o = ((ScriptableGameObj *)obj)->As_PhysicalGameObj();
	if (!o)
	{
		return 0;
	}
	unsigned char *x = (unsigned char *)o->Get_Anim_Control();
	if (!x)
	{
		return 0;
	}
	float a = (float)*(float *)(x+0x14);
	return a;
}

const char SCRIPTS_API *Get_Physics(GameObject *obj)
{
	if (Is_Door(obj)) return "Is_Door";
	else if (Is_Elevator(obj)) return "Is_Elevator";
	else if (Is_DamageableStaticPhys(obj)) return "Is_DamageableStaticPhys";
	else if (Is_AccessablePhys(obj)) return "Is_AccessablePhys";
	else if (Is_DecorationPhys(obj)) return "Is_DecorationPhys";
	else if (Is_HumanPhys(obj)) return "Is_HumanPhys";
	else if (Is_MotorVehicle(obj)) return "Is_MotorVehicle";
	else if (Is_Phys3(obj)) return "Is_Phys3";
	else if (Is_RigidBody(obj)) return "Is_RigidBody";
	else if (Is_ShakeableStatricPhys(obj)) return "Is_ShakeableStatricPhys";
	else if (Is_StaticAnimPhys(obj)) return "Is_StaticAnimPhys";
	else if (Is_StaticPhys(obj)) return "Is_StaticPhys";
	else if (Is_TimedDecorationPhys(obj)) return "Is_TimedDecorationPhys";
	else if (Is_VehiclePhys(obj)) return "Is_VehiclePhys";
	else if (Is_DynamicAnimPhys(obj)) return "Is_DynamicAnimPhys";
	else if (Is_BuildingAggregate(obj)) return "Is_BuildingAggregate";
	else if (Is_Projectile(obj)) return "Is_Projectile";
	else return "(null)";
}

void SCRIPTS_API Copy_Transform(GameObject *in,GameObject *out)
{
	if (!Commands->Get_ID(in) || !in || !Commands->Get_ID(out) || !out)
	{
		return;
	}
	PhysicalGameObj *i2 = ((ScriptableGameObj *)in)->As_PhysicalGameObj();
	if (!i2)
	{
		return;
	}
	PhysicalGameObj *o2 = ((ScriptableGameObj *)out)->As_PhysicalGameObj();
	if (!o2)
	{
		return;
	}
	o2->Set_Transform(i2->Get_Transform());
}

float SCRIPTS_API Get_Mass(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return 0;
	}
	PhysicalGameObj *o = (PhysicalGameObj *)As_PhysicalGameObj(obj);
	if (!o)
	{
		return 0;
	}
	PhysClass *phys = o->Peek_Physical_Object();
	return phys->As_MoveablePhysClass()->Get_Mass();
}

const char SCRIPTS_API *Get_Htree_Name(GameObject *obj)
{
	if (!obj || !Commands->Get_ID(obj))
	{
		return "DUMMY";
	}
	PhysicalGameObj *o = (PhysicalGameObj *)As_PhysicalGameObj(obj);
	if (!o)
	{
		return "DUMMY";
	}
	return o->Peek_Physical_Object()->Peek_Model()->Get_HTree()->Get_Name();
}

char SCRIPTS_API Get_Sex(GameObject *obj)
{
	return Get_Htree_Name(obj)[2];
}

void SCRIPTS_API Create_Effect_All_Stealthed_Objects_Area(const Vector3 &Position,float Distance,const char *object,const Vector3 &offset,int team)
{
	SLNode<BaseGameObj> *x = GameObjManager::GameObjList.Head();
	while (x) 
	{
		GameObject *obj = (GameObject *)x->Data();
		if (obj && As_ScriptableGameObj(obj))
		{
			Vector3 ObjPosition = Commands->Get_Position(obj);
			Vector3 TestPosition = Position;
			ObjPosition.Z = 0;
			TestPosition.Z = 0;
			if ((Commands->Get_Distance(ObjPosition,TestPosition) <= Distance))
			{
				if ((Get_Object_Type(obj) == team) || (team == 2))
				{
					bool stealth = Is_Stealth(obj);
					bool underground = obj->As_VehicleGameObj()->Is_Underground();
					if (stealth || underground)
					{
						ObjPosition = Commands->Get_Position(obj);
						ObjPosition.X += offset.X;
						ObjPosition.Y += offset.Y;
						ObjPosition.Z += offset.Z;
						Commands->Create_Object(object,ObjPosition);
					}
				}
			}
		}
		x = x->Next();
	}
}

void SCRIPTS_API Create_Effect_All_Of_Preset(const char *object,const char *preset,float ZAdjust,bool ZSet)
{
	SLNode<SmartGameObj> *x = GameObjManager::SmartGameObjList.Head();
	while (x) 
	{
		GameObject *obj = x->Data();
		if (obj)
		{
			if (!_stricmp(preset,Commands->Get_Preset_Name(obj)))
			{
				Vector3 ObjPosition = Commands->Get_Position(obj);
				if (ZSet)
				{
					ObjPosition.Z = ZAdjust;
				}
				else
				{
					ObjPosition.Z += ZAdjust;
				}
				GameObject *o = Commands->Create_Object(object,ObjPosition);
				Commands->Set_Facing(o,Commands->Get_Facing(obj));
			}
		}
		x = x->Next();
	}
}
