/*	Renegade Scripts.dll
	Copyright 2011 Tiberian Technologies

	This file is part of the Renegade scripts.dll
	The Renegade scripts.dll is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
	In addition, an exemption is given to allow Run Time Dynamic Linking of this code with any closed source module that does not contain code covered by this licence.
	Only the source code to the module(s) containing the licenced code has to be released.
*/
#include "general.h"
#include "gmlog.h"
#include "engine.h"
bool init = false;
extern bool gamelog;
struct Connection {
	SOCKET socket;
	char cbuffer[512];
	int cbufferFilled;
};
SimpleDynVecClass<Connection *> Connections;
SOCKET s;
void SSGMGameLog::Init(unsigned short port)
{
	if (port != 0)
	{
		WSADATA wsaData;
#pragma warning(suppress:6031)
		WSAStartup(MAKEWORD(1,1), &wsaData);
		s = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
		sockaddr_in SockAddr;
		SockAddr.sin_family = AF_INET;
		SockAddr.sin_addr.s_addr = INADDR_ANY;
		SockAddr.sin_port = htons(port);
		u_long i = 1;
		ioctlsocket(s,FIONBIO,&i);
		bind(s,(sockaddr *)&SockAddr,sizeof(SockAddr));
		listen(s,3);
		init = true;
	}
	else
	{
		init = false;
	}
}

void SSGMGameLog::Shutdown()
{
	if (init)
	{
		for (int i = 0;i < Connections.Count();i++)
		{
			closesocket(Connections[i]->socket);
		}
		closesocket(s);
		init = false;
	}
}

void SSGMGameLog::Think()
{
	SOCKET so = accept(s,NULL,NULL);
	if (so != INVALID_SOCKET)
	{
		Connection *c = new Connection;
		c->cbufferFilled = 0;
		c->socket = so;
		Connections.Add(c);
	}
	for (int i = 0;i < Connections.Count();i++)
	{
		char* bufferPtr = Connections[i]->cbuffer + Connections[i]->cbufferFilled;
		int chars_read = recv(Connections[i]->socket, bufferPtr, sizeof(Connections[i]->cbuffer)-Connections[i]->cbufferFilled,0);
		if( !(chars_read > 0) && WSAGetLastError() != WSAEWOULDBLOCK )
		{
			// Bot has closed the socket or something has gone wrong, you should deal with that here
			closesocket(Connections[i]->socket);
			Connections.Delete(Connections[i]);
			continue;
		}
		if (!(chars_read > 0))
		{
			continue;
		}
		Connections[i]->cbufferFilled += chars_read;
		// Look for complete messages in buffer
		char *startPos = Connections[i]->cbuffer;
		char *endPos = Connections[i]->cbuffer;
		endPos = strchr ( startPos, '\n' );
		while ( (endPos) && (Connections[i]->cbufferFilled > 0) )
		{
			// Copy input to message
			int messageLength = (endPos-startPos);
			char* message = new char[messageLength+1];
			strncpy ( message, startPos, messageLength );
			message[messageLength] = '\0';
			// Call a function to do something with message
			Console_Input(message);
			// Clean up message
			delete [] message;
			Connections[i]->cbufferFilled -= (++endPos - startPos);
			startPos = endPos;
			endPos = strchr ( startPos, '\n' );
		}
		memmove(Connections[i]->cbuffer,startPos,Connections[i]->cbufferFilled);
	}
}

void SSGMGameLog::Log_Message(const char *message,const char *category)
{
	if (init)
	{
		char time[256];
		memset(time,0,sizeof(time));
		time[0] = 0x3F;
		GetTimeFormat(LOCALE_SYSTEM_DEFAULT,TIME_FORCE24HOURFORMAT,0,"'['HH':'mm':'ss'] '",time,0xFF);
		StringClass str = "000";
		str += time;
		str += category;
		str += "";
		str += message;
		Send(str.Peek_Buffer());
	}
}

void SSGMGameLog::Log_Gamelog(const char *format,...)
{
	if (init && gamelog)
	{
		char time[256];
		memset(time,0,sizeof(time));
		time[0] = 0x3F;
		GetTimeFormat(LOCALE_SYSTEM_DEFAULT,TIME_FORCE24HOURFORMAT,0,"'['HH':'mm':'ss'] '",time,0xFF);
		StringClass str = "001";
		str += time;
		StringClass str2;
		va_list arg_list;
		va_start(arg_list,format);
		str2.Format_Args(format,arg_list);
		va_end(arg_list);
		str += str2;
		Send(str.Peek_Buffer());
	}
}

void SSGMGameLog::Log_RenLog(const char *message)
{
	if (init)
	{
		char time[256];
		memset(time,0,sizeof(time));
		time[0] = 0x3F;
		GetTimeFormat(LOCALE_SYSTEM_DEFAULT,TIME_FORCE24HOURFORMAT,0,"'['HH':'mm':'ss'] '",time,0xFF);
		StringClass str = "002";
		str += time;
		str += message;
		Send(str.Peek_Buffer());
	}
}

void SSGMGameLog::Send_Console(const char *message)
{
	if (init)
	{
		StringClass str = "003";
		str += message;
		Send(str.Peek_Buffer());
	}
}

void SSGMGameLog::Log_Custom(int id,const char *format,...)
{
	if (init)
	{
		StringClass str;
		str.Format("%03d\n",id);
		StringClass str2;
		va_list arg_list;
		va_start(arg_list,format);
		str2.Format_Args(format,arg_list);
		va_end(arg_list);
		str += str2;
		Send(str.Peek_Buffer());
	}
}

void SSGMGameLog::Send(const char *data)
{
	for (int i = 0;i < Connections.Count();i++)
	{
		send(Connections[i]->socket,data,strlen(data)+1,0);
	}
}
